/*---------------------------------------------------------------------------*\

  Copyright (C) 1999 by Jonathan Ringle.  You may freely use and modify
  this software, subject to the following restrictions:
  (1) This copyright notice must remain in the source, unchanged.
  (2) You are NOT allowed to redistribute this software in any form.
  
    NOTE: This software is provided without warranty of any kind.
    
\*---------------------------------------------------------------------------*/

#if !defined(__JR_SINGLETON_I__INCLUDED__)
#define __JR_SINGLETON_I__INCLUDED__

#if _MSC_VER >= 1000
#  pragma once
#endif // _MSC_VER >= 1000

#if !defined(__JR_SINGLETON_H__INCLUDED__)
#include "jr/singleton.h"
#endif

#include "jr/sync.h"

namespace jr
{
  template< class T > Singleton< T >*   Singleton< T >::_instance = 0;
  
  template< class T >
    Singleton< T >::Singleton()
    : Singleton_Base(),
    _object()
  {
  }
  
  template< class T >
    Singleton< T >::~Singleton()
  {
  }
  
  template< class T >
    inline T& Singleton< T >::instance()
  {
    if( !_instance )
    {
      Lock_Guard< Critical_Section > gate( manager_key() );
      
      if( !_instance ) // Double-checked locking
      {
        _instance = new Singleton< T >;
        add_to_manager( _instance ); // Singleton_Manager is responsible for deleting _instance
      }
    }
    
    return _instance->_object;
  }
  
} // namespace jr

#endif // !defined(__JR_SINGLETON_I__INCLUDED__)
